package main

import (
	"database/sql"
	"encoding/json"
	"fmt"
	"log"
	"net/http"
	"os"
	"time"

	_ "github.com/go-sql-driver/mysql"
)

// Location represents the JSON payload structure
type Location struct {
	Type string  `json:"_type"`
	Tst  int64   `json:"tst"`
	Lat  float64 `json:"lat"`
	Lon  float64 `json:"lon"`
	Tid  string  `json:"tid"`
	Batt int     `json:"batt"`
	Vac  int     `json:"vac"`
}

func init() {
	// Konfigurieren Sie den Standard-Logger
	log.SetOutput(os.Stdout)
	log.SetFlags(log.Ldate | log.Ltime | log.Lmicroseconds | log.Lshortfile)
}

func main() {
	log.Println("Server started. Listening on :8080")

	http.HandleFunc("/", handleRequest)
	http.ListenAndServe(":8080", nil)
}

func handleRequest(w http.ResponseWriter, r *http.Request) {
	log.Println("Handling request")

	if r.Method != http.MethodPost {
		log.Println("Method not allowed")
		http.Error(w, "Method not allowed", http.StatusMethodNotAllowed)
		return
	}

	// Extract values from headers
	device := r.Header.Get("X-Limit-D")
	user := r.Header.Get("X-Limit-U")

	log.Printf("Received request with device=%s and user=%s", device, user)

	var loc Location
	decoder := json.NewDecoder(r.Body)
	err := decoder.Decode(&loc)
	if err != nil {
		log.Printf("Invalid JSON payload: %v", err)
		http.Error(w, "Invalid JSON payload", http.StatusBadRequest)
		return
	}

	log.Printf("Received JSON payload: %+v", loc)

	if loc.Type == "location" {
		db, err := sql.Open("mysql", getDBConnectionString())
		if err != nil {
			log.Printf("Database connection error: %v", err)
			http.Error(w, fmt.Sprintf("Database connection error: %v", err), http.StatusInternalServerError)
			return
		}
		defer db.Close()

		dt := time.Unix(loc.Tst, 0).Format("2006-01-02 15:04:05")

		_, err = db.Exec("INSERT INTO locations (dt, tid, lat, lon, batt, vac, device, user) VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
			dt, loc.Tid, loc.Lat, loc.Lon, loc.Batt, loc.Vac, device, user)
		if err != nil {
			log.Printf("Database insertion error: %v", err)
			http.Error(w, fmt.Sprintf("Database insertion error: %v", err), http.StatusInternalServerError)
			return
		}

		log.Println("Location data inserted into the database")
	}

	response := make(map[string]interface{})
	// Optionally add objects to return to the app (e.g., friends or cards)
	json.NewEncoder(w).Encode(response)

	log.Println("Request handled successfully")
}

func getDBConnectionString() string {
	user := os.Getenv("DB_USER")
	password := os.Getenv("DB_PASSWORD")
	host := os.Getenv("DB_HOST")
	port := os.Getenv("DB_PORT")
	dbName := os.Getenv("DB_NAME")

	return fmt.Sprintf("%s:%s@tcp(%s:%s)/%s", user, password, host, port, dbName)
}
